//
// Written by Igor Tolmachev, IT Samples
//        mailto:support@itsamples.com
//
// Copyright (c) 2009.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name and all copyright 
// notices remains intact. If the source code in this file is used in 
// any  commercial application then a statement along the lines of 
// "Portions copyright (c) Igor Tolmachev, 2009" must be included in 
// the startup banner, "About" box or printed documentation. An email 
// letting me know that you are using it would be nice as well. That's 
// not much to ask considering the amount of work that went into this.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
// Expect bugs!
// 
// Please use and enjoy, and let me know of any bugs/mods/improvements 
// that you have found/implemented and I will fix/incorporate them into 
// this file. 

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Microsoft.Win32;
using System.Drawing.Imaging;
using System.IO;
using System.Threading;

namespace ExifDataChanger
{
	#region Public Delegates

	public delegate void DelegateAddThumbNail(ThumbNail thumbnail);

	#endregion
	/// <summary>
	/// Summary description for FormMain.
	/// </summary>
	public class FormMain : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Panel panelPhotoThumbs;
		private System.Windows.Forms.MenuItem menuFile;
		private System.Windows.Forms.MenuItem menuFileOpen;
		private System.Windows.Forms.MenuItem menuFileOpenFolder;
		private System.Windows.Forms.MenuItem menuHelpAbout;
		private System.Windows.Forms.MainMenu menuMain;
		private System.Windows.Forms.ColumnHeader columnHeaderTag;
		private System.Windows.Forms.RadioButton radioCreation;
		private ExifDataChanger.ButtonXP buttonChange;
		private System.Windows.Forms.Label labelPictureCaption;
		private System.Windows.Forms.RadioButton radioLastAccess;
		private System.Windows.Forms.RadioButton radioCustomDate;
		private System.Windows.Forms.ColumnHeader columnHeaderValue;

		private int m_normalLeft = 0;
		private int m_normalTop = 0;
		private int m_normalWidth = 0;
		private int m_normalHeight = 0;
		private int m_nMargin = 5;
		private int m_nLanguage = 0;
		private string m_strFilePath = "";
		private ArrayList m_arFiles = new ArrayList();
		private Thread m_pConvertThread = null;
		public DelegateAddThumbNail m_DelegateAddThumbNail;

		private bool m_bUseDateTimeCreated = true;
		private bool m_bUseDateTimeOriginal = true;
		private bool m_bUseDateTimeDigitized = true;
		private bool m_bUpdateFileDate = true;
		private bool m_bCreateBackups = true;

		private System.Windows.Forms.MenuItem menuHelp;
		private ExifDataChanger.FlatTextBox textCreation;
		private ExifDataChanger.FlatTextBox textLastAccess;
		private ExifDataChanger.FlatTextBox textLastWrite;
		private ExifDataChanger.FlatDateTimePicker dateCustomTimePicker;
		private System.Windows.Forms.OpenFileDialog openFileDialog;
		private FormWindowState m_windowState;
		private System.Windows.Forms.ListView listView;
		private ListViewColumnSorter lvwColumnSorter;
		private System.Windows.Forms.RadioButton radioLastWrite;
		private System.Windows.Forms.FolderBrowserDialog folderBrowserDialog;
		private System.Windows.Forms.MenuItem menuSeparator1;
		private System.Windows.Forms.ImageList menuIcons;
		private ExifDataChanger.MenuImage menuImages;
		private System.ComponentModel.IContainer components;
		private ExifDataChanger.StatusBarProgressPanel statusBarProgressPane;
		private ExifDataChanger.StatusBarInfoPanel statusBarInfoPane;
		private System.Windows.Forms.StatusBar statusBar;
		private System.Windows.Forms.Timer timerTips;
		private System.Windows.Forms.MenuItem menuFileUpdate;
		private System.Windows.Forms.MenuItem menuInsertDTCreated;
		private System.Windows.Forms.MenuItem menuInsertDTOriginal;
		private System.Windows.Forms.MenuItem menuInsertDTDigitized;
		private System.Windows.Forms.MenuItem menuDeleteDTCreated;
		private System.Windows.Forms.MenuItem menuDeleteDTOriginal;
		private System.Windows.Forms.MenuItem menuDeleteDTDigitized;
		private System.Windows.Forms.MenuItem menuSeparator3;
		private System.Windows.Forms.MenuItem menuFileInsert;
		private System.Windows.Forms.MenuItem menuFileDelete;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem menuSettings;
		private System.Windows.Forms.MenuItem menuOptions;
		private System.Windows.Forms.MenuItem menuLanguage;
		private System.Windows.Forms.MenuItem menuLanguageEnglish;
		private System.Windows.Forms.MenuItem menuLanguageRussian;
		private System.Windows.Forms.MenuItem menuHelpManual;
		private System.Windows.Forms.MenuItem menuSeparator2;
		private System.Windows.Forms.MenuItem menuSeparator4;
		private System.Windows.Forms.MenuItem menuFileExit;

		#region Properties
		public int Language
		{
			get { return m_nLanguage; }
		}
		#endregion

		public FormMain()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// the filter and filter index are set here just to make it easier to read and change.
			// these properties can also be set in Design mode
			this.openFileDialog.Filter = "JPEG files (*.jpg,*jpeg,*.jpe,*.jfif)|*.jpg;*.jpeg;*.jpe;*.jfif|All Image Files|*.*";
			this.openFileDialog.AddExtension = true;
			lvwColumnSorter = new ListViewColumnSorter();
			this.listView.ListViewItemSorter = lvwColumnSorter;
			
			this.statusBar.DrawItem += new StatusBarDrawItemEventHandler(statusBarProgressPane.ParentDrawItemHandler);
			this.statusBar.DrawItem += new StatusBarDrawItemEventHandler(statusBarInfoPane.ParentDrawItemHandler);
			statusBarProgressPane.ProgressBar.Minimum = 0;
			statusBarProgressPane.ProgressBar.Maximum = 100;
			// Also copy over the value
			statusBarProgressPane.ProgressBar.Value = 0;
			timerTips.Interval = 3000;
			timerTips.Stop();
		
			// initialize delegates
			m_DelegateAddThumbNail = new DelegateAddThumbNail(this.AddThumbNail);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				m_arFiles.Clear();
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FormMain));
			this.panelPhotoThumbs = new System.Windows.Forms.Panel();
			this.menuFile = new System.Windows.Forms.MenuItem();
			this.menuFileOpen = new System.Windows.Forms.MenuItem();
			this.menuFileOpenFolder = new System.Windows.Forms.MenuItem();
			this.menuSeparator1 = new System.Windows.Forms.MenuItem();
			this.menuFileInsert = new System.Windows.Forms.MenuItem();
			this.menuInsertDTCreated = new System.Windows.Forms.MenuItem();
			this.menuInsertDTOriginal = new System.Windows.Forms.MenuItem();
			this.menuInsertDTDigitized = new System.Windows.Forms.MenuItem();
			this.menuFileDelete = new System.Windows.Forms.MenuItem();
			this.menuDeleteDTCreated = new System.Windows.Forms.MenuItem();
			this.menuDeleteDTOriginal = new System.Windows.Forms.MenuItem();
			this.menuDeleteDTDigitized = new System.Windows.Forms.MenuItem();
			this.menuFileUpdate = new System.Windows.Forms.MenuItem();
			this.menuSeparator3 = new System.Windows.Forms.MenuItem();
			this.menuFileExit = new System.Windows.Forms.MenuItem();
			this.menuHelp = new System.Windows.Forms.MenuItem();
			this.menuHelpManual = new System.Windows.Forms.MenuItem();
			this.menuSeparator4 = new System.Windows.Forms.MenuItem();
			this.menuHelpAbout = new System.Windows.Forms.MenuItem();
			this.menuMain = new System.Windows.Forms.MainMenu();
			this.menuSettings = new System.Windows.Forms.MenuItem();
			this.menuOptions = new System.Windows.Forms.MenuItem();
			this.menuSeparator2 = new System.Windows.Forms.MenuItem();
			this.menuLanguage = new System.Windows.Forms.MenuItem();
			this.menuLanguageEnglish = new System.Windows.Forms.MenuItem();
			this.menuLanguageRussian = new System.Windows.Forms.MenuItem();
			this.listView = new System.Windows.Forms.ListView();
			this.columnHeaderTag = new System.Windows.Forms.ColumnHeader();
			this.columnHeaderValue = new System.Windows.Forms.ColumnHeader();
			this.radioCreation = new System.Windows.Forms.RadioButton();
			this.radioLastWrite = new System.Windows.Forms.RadioButton();
			this.radioLastAccess = new System.Windows.Forms.RadioButton();
			this.radioCustomDate = new System.Windows.Forms.RadioButton();
			this.textCreation = new ExifDataChanger.FlatTextBox();
			this.textLastWrite = new ExifDataChanger.FlatTextBox();
			this.textLastAccess = new ExifDataChanger.FlatTextBox();
			this.buttonChange = new ExifDataChanger.ButtonXP();
			this.dateCustomTimePicker = new ExifDataChanger.FlatDateTimePicker();
			this.labelPictureCaption = new System.Windows.Forms.Label();
			this.openFileDialog = new System.Windows.Forms.OpenFileDialog();
			this.folderBrowserDialog = new System.Windows.Forms.FolderBrowserDialog();
			this.menuIcons = new System.Windows.Forms.ImageList(this.components);
			this.menuImages = new ExifDataChanger.MenuImage(this.components);
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.statusBar = new System.Windows.Forms.StatusBar();
			this.statusBarInfoPane = new ExifDataChanger.StatusBarInfoPanel();
			this.statusBarProgressPane = new ExifDataChanger.StatusBarProgressPanel();
			this.timerTips = new System.Windows.Forms.Timer(this.components);
			((System.ComponentModel.ISupportInitialize)(this.statusBarInfoPane)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarProgressPane)).BeginInit();
			this.SuspendLayout();
			// 
			// panelPhotoThumbs
			// 
			this.panelPhotoThumbs.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.panelPhotoThumbs.AutoScroll = true;
			this.panelPhotoThumbs.BackColor = System.Drawing.Color.LightSteelBlue;
			this.panelPhotoThumbs.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.panelPhotoThumbs.Cursor = System.Windows.Forms.Cursors.Default;
			this.panelPhotoThumbs.ForeColor = System.Drawing.Color.CornflowerBlue;
			this.panelPhotoThumbs.Location = new System.Drawing.Point(0, 1);
			this.panelPhotoThumbs.Name = "panelPhotoThumbs";
			this.panelPhotoThumbs.Size = new System.Drawing.Size(338, 432);
			this.panelPhotoThumbs.TabIndex = 0;
			// 
			// menuFile
			// 
			this.menuFile.Index = 0;
			this.menuFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.menuFileOpen,
																					 this.menuFileOpenFolder,
																					 this.menuSeparator1,
																					 this.menuFileInsert,
																					 this.menuFileDelete,
																					 this.menuFileUpdate,
																					 this.menuSeparator3,
																					 this.menuFileExit});
			this.menuFile.Text = "&File";
			// 
			// menuFileOpen
			// 
			this.menuFileOpen.Index = 0;
			this.menuImages.SetMenuImage(this.menuFileOpen, "0");
			this.menuFileOpen.OwnerDraw = true;
			this.menuFileOpen.Shortcut = System.Windows.Forms.Shortcut.CtrlO;
			this.menuFileOpen.Text = "Open File";
			this.menuFileOpen.Click += new System.EventHandler(this.OnFileOpen);
			this.menuFileOpen.Select += new System.EventHandler(this.OnSelectFileOpen);
			// 
			// menuFileOpenFolder
			// 
			this.menuFileOpenFolder.Index = 1;
			this.menuImages.SetMenuImage(this.menuFileOpenFolder, "1");
			this.menuFileOpenFolder.OwnerDraw = true;
			this.menuFileOpenFolder.Shortcut = System.Windows.Forms.Shortcut.CtrlShiftO;
			this.menuFileOpenFolder.Text = "Open Folder";
			this.menuFileOpenFolder.Click += new System.EventHandler(this.OnFolderOpen);
			this.menuFileOpenFolder.Select += new System.EventHandler(this.OnSelectFolderOpen);
			// 
			// menuSeparator1
			// 
			this.menuSeparator1.Index = 2;
			this.menuImages.SetMenuImage(this.menuSeparator1, null);
			this.menuSeparator1.OwnerDraw = true;
			this.menuSeparator1.Text = "-";
			// 
			// menuFileInsert
			// 
			this.menuFileInsert.Enabled = false;
			this.menuFileInsert.Index = 3;
			this.menuImages.SetMenuImage(this.menuFileInsert, "5");
			this.menuFileInsert.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						   this.menuInsertDTCreated,
																						   this.menuInsertDTOriginal,
																						   this.menuInsertDTDigitized});
			this.menuFileInsert.OwnerDraw = true;
			this.menuFileInsert.Text = "Insert Tag";
			// 
			// menuInsertDTCreated
			// 
			this.menuInsertDTCreated.Index = 0;
			this.menuImages.SetMenuImage(this.menuInsertDTCreated, null);
			this.menuInsertDTCreated.OwnerDraw = true;
			this.menuInsertDTCreated.Text = "DateTime Created";
			this.menuInsertDTCreated.Click += new System.EventHandler(this.OnInsertDTCreated);
			this.menuInsertDTCreated.Select += new System.EventHandler(this.OnSelectInsertDTCreated);
			// 
			// menuInsertDTOriginal
			// 
			this.menuInsertDTOriginal.Index = 1;
			this.menuImages.SetMenuImage(this.menuInsertDTOriginal, null);
			this.menuInsertDTOriginal.OwnerDraw = true;
			this.menuInsertDTOriginal.Text = "DateTime Original";
			this.menuInsertDTOriginal.Click += new System.EventHandler(this.OnInsertDTOriginal);
			this.menuInsertDTOriginal.Select += new System.EventHandler(this.OnSelectInsertDTOriginal);
			// 
			// menuInsertDTDigitized
			// 
			this.menuInsertDTDigitized.Index = 2;
			this.menuImages.SetMenuImage(this.menuInsertDTDigitized, null);
			this.menuInsertDTDigitized.OwnerDraw = true;
			this.menuInsertDTDigitized.Text = "DateTime Digitized";
			this.menuInsertDTDigitized.Click += new System.EventHandler(this.OnInsertDTDigitized);
			this.menuInsertDTDigitized.Select += new System.EventHandler(this.OnSelectInsertDTDigitized);
			// 
			// menuFileDelete
			// 
			this.menuFileDelete.Enabled = false;
			this.menuFileDelete.Index = 4;
			this.menuImages.SetMenuImage(this.menuFileDelete, "6");
			this.menuFileDelete.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						   this.menuDeleteDTCreated,
																						   this.menuDeleteDTOriginal,
																						   this.menuDeleteDTDigitized});
			this.menuFileDelete.OwnerDraw = true;
			this.menuFileDelete.Text = "Delete Tag";
			// 
			// menuDeleteDTCreated
			// 
			this.menuDeleteDTCreated.Index = 0;
			this.menuImages.SetMenuImage(this.menuDeleteDTCreated, null);
			this.menuDeleteDTCreated.OwnerDraw = true;
			this.menuDeleteDTCreated.Text = "DateTime Created";
			this.menuDeleteDTCreated.Click += new System.EventHandler(this.OnDeleteDTCreated);
			this.menuDeleteDTCreated.Select += new System.EventHandler(this.OnSelectDeleteDTCreated);
			// 
			// menuDeleteDTOriginal
			// 
			this.menuDeleteDTOriginal.Index = 1;
			this.menuImages.SetMenuImage(this.menuDeleteDTOriginal, null);
			this.menuDeleteDTOriginal.OwnerDraw = true;
			this.menuDeleteDTOriginal.Text = "DateTime Original";
			this.menuDeleteDTOriginal.Click += new System.EventHandler(this.OnDeleteDTOriginal);
			this.menuDeleteDTOriginal.Select += new System.EventHandler(this.OnSelectDeleteDTOriginal);
			// 
			// menuDeleteDTDigitized
			// 
			this.menuDeleteDTDigitized.Index = 2;
			this.menuImages.SetMenuImage(this.menuDeleteDTDigitized, null);
			this.menuDeleteDTDigitized.OwnerDraw = true;
			this.menuDeleteDTDigitized.Text = "DateTime Digitized";
			this.menuDeleteDTDigitized.Click += new System.EventHandler(this.OnDeleteDTDigitized);
			this.menuDeleteDTDigitized.Select += new System.EventHandler(this.OnSelectDeleteDTDigitized);
			// 
			// menuFileUpdate
			// 
			this.menuFileUpdate.Enabled = false;
			this.menuFileUpdate.Index = 5;
			this.menuImages.SetMenuImage(this.menuFileUpdate, "3");
			this.menuFileUpdate.OwnerDraw = true;
			this.menuFileUpdate.Text = "Update File Date";
			this.menuFileUpdate.Click += new System.EventHandler(this.OnFileUpdate);
			this.menuFileUpdate.Select += new System.EventHandler(this.OnSelectFileUpdate);
			// 
			// menuSeparator3
			// 
			this.menuSeparator3.Index = 6;
			this.menuImages.SetMenuImage(this.menuSeparator3, null);
			this.menuSeparator3.OwnerDraw = true;
			this.menuSeparator3.Text = "-";
			// 
			// menuFileExit
			// 
			this.menuFileExit.Index = 7;
			this.menuImages.SetMenuImage(this.menuFileExit, "4");
			this.menuFileExit.OwnerDraw = true;
			this.menuFileExit.Shortcut = System.Windows.Forms.Shortcut.CtrlQ;
			this.menuFileExit.Text = "Exit";
			this.menuFileExit.Click += new System.EventHandler(this.OnFileExit);
			this.menuFileExit.Select += new System.EventHandler(this.OnSelectExit);
			// 
			// menuHelp
			// 
			this.menuHelp.Index = 2;
			this.menuHelp.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.menuHelpManual,
																					 this.menuSeparator4,
																					 this.menuHelpAbout});
			this.menuHelp.Text = "&Help";
			// 
			// menuHelpManual
			// 
			this.menuHelpManual.Index = 0;
			this.menuImages.SetMenuImage(this.menuHelpManual, "9");
			this.menuHelpManual.OwnerDraw = true;
			this.menuHelpManual.Text = "Manual";
			this.menuHelpManual.Click += new System.EventHandler(this.OnHelp);
			this.menuHelpManual.Select += new System.EventHandler(this.OnSelectManual);
			// 
			// menuSeparator4
			// 
			this.menuSeparator4.Index = 1;
			this.menuImages.SetMenuImage(this.menuSeparator4, null);
			this.menuSeparator4.OwnerDraw = true;
			this.menuSeparator4.Text = "-";
			// 
			// menuHelpAbout
			// 
			this.menuHelpAbout.Index = 2;
			this.menuImages.SetMenuImage(this.menuHelpAbout, "7");
			this.menuHelpAbout.OwnerDraw = true;
			this.menuHelpAbout.Text = "About program";
			this.menuHelpAbout.Click += new System.EventHandler(this.OnHelpAbout);
			this.menuHelpAbout.Select += new System.EventHandler(this.OnSelectAbout);
			// 
			// menuMain
			// 
			this.menuMain.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.menuFile,
																					 this.menuSettings,
																					 this.menuHelp});
			// 
			// menuSettings
			// 
			this.menuSettings.Index = 1;
			this.menuSettings.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						 this.menuOptions,
																						 this.menuSeparator2,
																						 this.menuLanguage});
			this.menuSettings.Text = "&Settings";
			// 
			// menuOptions
			// 
			this.menuOptions.Index = 0;
			this.menuImages.SetMenuImage(this.menuOptions, "2");
			this.menuOptions.OwnerDraw = true;
			this.menuOptions.Text = "Options";
			this.menuOptions.Click += new System.EventHandler(this.OnOptions);
			this.menuOptions.Select += new System.EventHandler(this.OnSelectOptions);
			// 
			// menuSeparator2
			// 
			this.menuSeparator2.Index = 1;
			this.menuImages.SetMenuImage(this.menuSeparator2, null);
			this.menuSeparator2.OwnerDraw = true;
			this.menuSeparator2.Text = "-";
			// 
			// menuLanguage
			// 
			this.menuLanguage.Index = 2;
			this.menuImages.SetMenuImage(this.menuLanguage, "8");
			this.menuLanguage.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						 this.menuLanguageEnglish,
																						 this.menuLanguageRussian});
			this.menuLanguage.OwnerDraw = true;
			this.menuLanguage.Text = "Language";
			this.menuLanguage.Select += new System.EventHandler(this.OnSelectLanguage);
			// 
			// menuLanguageEnglish
			// 
			this.menuLanguageEnglish.Checked = true;
			this.menuLanguageEnglish.Index = 0;
			this.menuImages.SetMenuImage(this.menuLanguageEnglish, null);
			this.menuLanguageEnglish.OwnerDraw = true;
			this.menuLanguageEnglish.Text = "English";
			this.menuLanguageEnglish.Click += new System.EventHandler(this.OnLanguageEnglish);
			// 
			// menuLanguageRussian
			// 
			this.menuLanguageRussian.Index = 1;
			this.menuImages.SetMenuImage(this.menuLanguageRussian, null);
			this.menuLanguageRussian.OwnerDraw = true;
			this.menuLanguageRussian.Text = "Russian";
			this.menuLanguageRussian.Click += new System.EventHandler(this.OnLanguageRussian);
			// 
			// listView
			// 
			this.listView.AllowColumnReorder = true;
			this.listView.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.listView.BackColor = System.Drawing.Color.Cornsilk;
			this.listView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																					   this.columnHeaderTag,
																					   this.columnHeaderValue});
			this.listView.ForeColor = System.Drawing.Color.Navy;
			this.listView.FullRowSelect = true;
			this.listView.GridLines = true;
			this.listView.Location = new System.Drawing.Point(341, 1);
			this.listView.MultiSelect = false;
			this.listView.Name = "listView";
			this.listView.Size = new System.Drawing.Size(291, 249);
			this.listView.Sorting = System.Windows.Forms.SortOrder.Ascending;
			this.listView.TabIndex = 6;
			this.listView.View = System.Windows.Forms.View.Details;
			this.listView.Click += new System.EventHandler(this.OnRowClick);
			this.listView.ColumnClick += new System.Windows.Forms.ColumnClickEventHandler(this.OnColumnClick);
			// 
			// columnHeaderTag
			// 
			this.columnHeaderTag.Text = "  Property";
			this.columnHeaderTag.Width = 100;
			// 
			// columnHeaderValue
			// 
			this.columnHeaderValue.Text = "  Value";
			this.columnHeaderValue.Width = 187;
			// 
			// radioCreation
			// 
			this.radioCreation.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.radioCreation.Location = new System.Drawing.Point(354, 289);
			this.radioCreation.Name = "radioCreation";
			this.radioCreation.Size = new System.Drawing.Size(118, 17);
			this.radioCreation.TabIndex = 7;
			this.radioCreation.Text = "Creation Date";
			// 
			// radioLastWrite
			// 
			this.radioLastWrite.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.radioLastWrite.Location = new System.Drawing.Point(354, 314);
			this.radioLastWrite.Name = "radioLastWrite";
			this.radioLastWrite.Size = new System.Drawing.Size(118, 17);
			this.radioLastWrite.TabIndex = 8;
			this.radioLastWrite.Text = "Last Write Date";
			// 
			// radioLastAccess
			// 
			this.radioLastAccess.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.radioLastAccess.Location = new System.Drawing.Point(354, 339);
			this.radioLastAccess.Name = "radioLastAccess";
			this.radioLastAccess.Size = new System.Drawing.Size(118, 17);
			this.radioLastAccess.TabIndex = 9;
			this.radioLastAccess.Text = "Last Access Date";
			// 
			// radioCustomDate
			// 
			this.radioCustomDate.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.radioCustomDate.Location = new System.Drawing.Point(354, 364);
			this.radioCustomDate.Name = "radioCustomDate";
			this.radioCustomDate.Size = new System.Drawing.Size(118, 17);
			this.radioCustomDate.TabIndex = 10;
			this.radioCustomDate.Text = "Custom Date";
			// 
			// textCreation
			// 
			this.textCreation.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.textCreation.BackColor = System.Drawing.SystemColors.Window;
			this.textCreation.ForeColor = System.Drawing.SystemColors.WindowText;
			this.textCreation.Location = new System.Drawing.Point(482, 286);
			this.textCreation.Name = "textCreation";
			this.textCreation.ReadOnly = true;
			this.textCreation.Size = new System.Drawing.Size(137, 20);
			this.textCreation.TabIndex = 12;
			this.textCreation.TabStop = false;
			this.textCreation.Text = "";
			this.textCreation.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// textLastWrite
			// 
			this.textLastWrite.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.textLastWrite.BackColor = System.Drawing.SystemColors.Window;
			this.textLastWrite.ForeColor = System.Drawing.SystemColors.WindowText;
			this.textLastWrite.Location = new System.Drawing.Point(482, 312);
			this.textLastWrite.Name = "textLastWrite";
			this.textLastWrite.ReadOnly = true;
			this.textLastWrite.Size = new System.Drawing.Size(137, 20);
			this.textLastWrite.TabIndex = 13;
			this.textLastWrite.TabStop = false;
			this.textLastWrite.Text = "";
			this.textLastWrite.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// textLastAccess
			// 
			this.textLastAccess.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.textLastAccess.BackColor = System.Drawing.SystemColors.Window;
			this.textLastAccess.ForeColor = System.Drawing.SystemColors.WindowText;
			this.textLastAccess.Location = new System.Drawing.Point(482, 338);
			this.textLastAccess.Name = "textLastAccess";
			this.textLastAccess.ReadOnly = true;
			this.textLastAccess.Size = new System.Drawing.Size(137, 20);
			this.textLastAccess.TabIndex = 14;
			this.textLastAccess.TabStop = false;
			this.textLastAccess.Text = "";
			this.textLastAccess.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// buttonChange
			// 
			this.buttonChange.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.buttonChange.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(0)), ((System.Byte)(236)), ((System.Byte)(233)), ((System.Byte)(216)));
			this.buttonChange.DefaultScheme = true;
			this.buttonChange.DialogResult = System.Windows.Forms.DialogResult.None;
			this.buttonChange.Hint = "";
			this.buttonChange.Location = new System.Drawing.Point(429, 395);
			this.buttonChange.Name = "buttonChange";
			this.buttonChange.Scheme = ExifDataChanger.ButtonXP.Schemes.Blue;
			this.buttonChange.Size = new System.Drawing.Size(109, 27);
			this.buttonChange.TabIndex = 11;
			this.buttonChange.Text = "Change";
			this.buttonChange.Click += new System.EventHandler(this.OnButtonChange);
			// 
			// dateCustomTimePicker
			// 
			this.dateCustomTimePicker.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.dateCustomTimePicker.CustomFormat = "yyyy:MM:dd HH:mm:ss";
			this.dateCustomTimePicker.Format = System.Windows.Forms.DateTimePickerFormat.Custom;
			this.dateCustomTimePicker.Location = new System.Drawing.Point(481, 364);
			this.dateCustomTimePicker.Name = "dateCustomTimePicker";
			this.dateCustomTimePicker.Size = new System.Drawing.Size(139, 20);
			this.dateCustomTimePicker.TabIndex = 15;
			this.dateCustomTimePicker.Value = new System.DateTime(2005, 1, 1, 0, 0, 0, 0);
			// 
			// labelPictureCaption
			// 
			this.labelPictureCaption.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.labelPictureCaption.Font = new System.Drawing.Font("Verdana", 11.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.labelPictureCaption.Location = new System.Drawing.Point(341, 259);
			this.labelPictureCaption.Name = "labelPictureCaption";
			this.labelPictureCaption.Size = new System.Drawing.Size(291, 17);
			this.labelPictureCaption.TabIndex = 16;
			this.labelPictureCaption.Text = "Picture";
			this.labelPictureCaption.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// openFileDialog
			// 
			this.openFileDialog.FilterIndex = 7;
			this.openFileDialog.Multiselect = true;
			// 
			// folderBrowserDialog
			// 
			this.folderBrowserDialog.Description = "Select Picture files with EXIF data";
			this.folderBrowserDialog.ShowNewFolderButton = false;
			// 
			// menuIcons
			// 
			this.menuIcons.ColorDepth = System.Windows.Forms.ColorDepth.Depth24Bit;
			this.menuIcons.ImageSize = new System.Drawing.Size(16, 16);
			this.menuIcons.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("menuIcons.ImageStream")));
			this.menuIcons.TransparentColor = System.Drawing.Color.Magenta;
			// 
			// menuImages
			// 
			this.menuImages.ImageList = this.menuIcons;
			// 
			// menuItem1
			// 
			this.menuItem1.Index = -1;
			this.menuImages.SetMenuImage(this.menuItem1, null);
			this.menuItem1.OwnerDraw = true;
			this.menuItem1.Text = "";
			// 
			// statusBar
			// 
			this.statusBar.Location = new System.Drawing.Point(0, 431);
			this.statusBar.Name = "statusBar";
			this.statusBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																						 this.statusBarInfoPane,
																						 this.statusBarProgressPane});
			this.statusBar.ShowPanels = true;
			this.statusBar.Size = new System.Drawing.Size(632, 22);
			this.statusBar.TabIndex = 17;
			// 
			// statusBarInfoPane
			// 
			this.statusBarInfoPane.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.statusBarInfoPane.BorderStyle = System.Windows.Forms.StatusBarPanelBorderStyle.None;
			this.statusBarInfoPane.Style = System.Windows.Forms.StatusBarPanelStyle.OwnerDraw;
			this.statusBarInfoPane.Text = "Ready";
			this.statusBarInfoPane.Width = 466;
			// 
			// statusBarProgressPane
			// 
			this.statusBarProgressPane.Style = System.Windows.Forms.StatusBarPanelStyle.OwnerDraw;
			this.statusBarProgressPane.Text = "Ready";
			this.statusBarProgressPane.Width = 150;
			// 
			// timerTips
			// 
			this.timerTips.Tick += new System.EventHandler(this.OnTimerTips);
			// 
			// FormMain
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(632, 453);
			this.Controls.Add(this.statusBar);
			this.Controls.Add(this.labelPictureCaption);
			this.Controls.Add(this.dateCustomTimePicker);
			this.Controls.Add(this.textLastAccess);
			this.Controls.Add(this.textLastWrite);
			this.Controls.Add(this.textCreation);
			this.Controls.Add(this.buttonChange);
			this.Controls.Add(this.radioCustomDate);
			this.Controls.Add(this.radioLastAccess);
			this.Controls.Add(this.radioLastWrite);
			this.Controls.Add(this.radioCreation);
			this.Controls.Add(this.listView);
			this.Controls.Add(this.panelPhotoThumbs);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Menu = this.menuMain;
			this.MinimumSize = new System.Drawing.Size(640, 480);
			this.Name = "FormMain";
			this.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
			this.Text = "EXIF Date Changer";
			this.Resize += new System.EventHandler(this.OnResize);
			this.Closing += new System.ComponentModel.CancelEventHandler(this.OnClosing);
			this.Load += new System.EventHandler(this.OnLoad);
			this.Move += new System.EventHandler(this.OnMove);
			((System.ComponentModel.ISupportInitialize)(this.statusBarInfoPane)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarProgressPane)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new FormMain());
		}

		private void OnLoad(object sender, System.EventArgs e)
		{
			Rectangle rectScreen = SystemInformation.WorkingArea;
			
			// center screen
			m_normalLeft = (rectScreen.Width-640)/2;
			m_normalTop = (rectScreen.Height-480)/2;
			m_normalWidth = 640;
			m_normalHeight = 480;
			m_windowState = FormWindowState.Normal;
			
			// attempt to read state from registry
			RegistryKey hKey = Registry.CurrentUser.OpenSubKey(@"Software\ExifChanger");
			if(hKey != null)
			{
				m_nLanguage = (int)hKey.GetValue("Language",0);
				m_normalLeft = (int)hKey.GetValue("Left", this.Left);
				m_normalTop = (int)hKey.GetValue("Top", this.Top);
				m_normalWidth = (int)hKey.GetValue("Width", this.Width);
				m_normalHeight = (int)hKey.GetValue("Height", this.Height);
				m_windowState = (FormWindowState)hKey.GetValue("WindowState", (int)this.WindowState);
				
				int nValue = 0;

				nValue = (int)hKey.GetValue("DTCreated",1);
				if(nValue<1) m_bUseDateTimeCreated = false;
				nValue = (int)hKey.GetValue("DTDigitized",1);
				if(nValue<1) m_bUseDateTimeDigitized = false;
				nValue = (int)hKey.GetValue("DTOriginal",1);
				if(nValue<1) m_bUseDateTimeOriginal = false;
				nValue = (int)hKey.GetValue("FileDate",1);
				if(nValue<1) m_bUpdateFileDate = false;
				nValue = (int)hKey.GetValue("Backups",1);
				if(nValue<1) m_bCreateBackups = false;

				hKey.Close();
			}
			
			this.Location = new Point(m_normalLeft, m_normalTop);
			this.Size = new Size(m_normalWidth, m_normalHeight);
			this.WindowState = m_windowState;
			
			this.columnHeaderTag.Width = this.listView.Width/3;
			this.columnHeaderValue.Width = this.listView.Width - this.columnHeaderTag.Width - 2*SystemInformation.Border3DSize.Width;

			dateCustomTimePicker.Value = DateTime.Now;

			if(m_nLanguage>0)
			{
				this.menuLanguageEnglish.Checked = false;
				this.menuLanguageRussian.Checked = true;
			}
			else
			{
				this.menuLanguageEnglish.Checked = true;
				this.menuLanguageRussian.Checked = false;
			}

			SwitchLanguage();
		}
		
		private void OnResize(object sender, System.EventArgs e)
		{
			if(m_pConvertThread!=null && m_pConvertThread.IsAlive) return;

			// save width and height
			if(this.WindowState == FormWindowState.Normal)
			{
				m_normalWidth = this.Width;
				m_normalHeight = this.Height;
			}

			// move any thumbnails already present
			int nHorizontal = -1;
			int nControls = this.panelPhotoThumbs.Controls.Count;
			if(nControls>0)
			{
				for(int i=0; i<nControls; i++)
				{
					ThumbNail thumbnail = this.panelPhotoThumbs.Controls[i] as ThumbNail;
					if (nHorizontal < 0)
					{
						// determine how many thumbnails can be displayed on one row
						nHorizontal = (int)(this.panelPhotoThumbs.Width/(thumbnail.Width+2*m_nMargin));
					}
					if(thumbnail!=null)
					{
						// set the position for the thumbnail and add it to the panel's controls
						thumbnail.Left = m_nMargin + (thumbnail.Width+m_nMargin) * (i % nHorizontal);
						thumbnail.Top = m_nMargin + (thumbnail.Height+m_nMargin) * (i / nHorizontal);
						this.panelPhotoThumbs.Invalidate();
					}
				}
			}
		}

		private void OnMove(object sender, System.EventArgs e)
		{
			// save position
			if(this.WindowState == FormWindowState.Normal)
			{
				m_normalLeft = this.Left;
				m_normalTop = this.Top;
			}
		}
		
		private void OnClosing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if(m_pConvertThread != null && m_pConvertThread.IsAlive)
				m_pConvertThread.Abort();

			// save position, size and state
			RegistryKey hKey = Registry.CurrentUser.CreateSubKey(@"Software\ExifChanger");
			
			if(hKey!=null)
			{
				hKey.SetValue("Language", m_nLanguage);
				hKey.SetValue("Left", m_normalLeft);
				hKey.SetValue("Top", m_normalTop);
				hKey.SetValue("Width", m_normalWidth);
				hKey.SetValue("Height", m_normalHeight);

				m_windowState = this.WindowState;
				if(m_windowState == FormWindowState.Minimized)
					m_windowState = FormWindowState.Normal;
				hKey.SetValue("WindowState", (int)m_windowState);

				hKey.Close();
			}
		}

		private void OnFileOpen(object sender, System.EventArgs e)
		{
			if(this.openFileDialog.ShowDialog().Equals(DialogResult.OK))
			{
				m_arFiles.Clear();
				foreach(string strFile in openFileDialog.FileNames)
					m_arFiles.Add(strFile);
				
				// clean up any thumbnails already present
				while(this.panelPhotoThumbs.Controls.Count > 0)
				{
					ThumbNail thumbnail = this.panelPhotoThumbs.Controls[0] as ThumbNail;
					thumbnail.Dispose();
					this.panelPhotoThumbs.Controls.Remove(thumbnail);
				}

				m_pConvertThread = new Thread(new ThreadStart(LoadThumbnails));
				m_pConvertThread.Start();
			}
		}

		private void OnFolderOpen(object sender, System.EventArgs e)
		{
			if(this.folderBrowserDialog.ShowDialog().Equals(DialogResult.OK))
			{
				string strFolder = this.folderBrowserDialog.SelectedPath;
				string strExtension = "*.jpg;*.jpeg;*.jpe;*.jfif";

				DirectoryInfo directoryInfo = new DirectoryInfo(strFolder);
				
				FileInfo [] fileInfo = directoryInfo.GetFiles();
				m_arFiles.Clear();
				foreach(FileInfo file in fileInfo)
				{
					strExtension = file.Extension;
					strExtension = strExtension.ToLower();
					if(strExtension==".jpg" || strExtension==".jpeg" || strExtension==".jpe" || strExtension==".jfif")
						m_arFiles.Add(file.FullName);
				}

				// clean up any thumbnails already present
				while(this.panelPhotoThumbs.Controls.Count > 0)
				{
					ThumbNail thumbnail = this.panelPhotoThumbs.Controls[0] as ThumbNail;
					thumbnail.Dispose();
					this.panelPhotoThumbs.Controls.Remove(thumbnail);
				}

				m_pConvertThread = new Thread(new ThreadStart(LoadThumbnails));
				m_pConvertThread.Start();
			}
		}

		private void OnColumnClick(object sender, System.Windows.Forms.ColumnClickEventArgs e)
		{
			// Determine if clicked column is already the column that is being sorted.
			if (e.Column == lvwColumnSorter.SortColumn) 
			{
				// Reverse the current sort direction for this column.
				if (lvwColumnSorter.Order == SortOrder.Ascending) 
				{
					lvwColumnSorter.Order = SortOrder.Descending;
				}
				else 
				{
					lvwColumnSorter.Order = SortOrder.Ascending;
				}
			}
			else 
			{
				// Set the column number that is to be sorted; default to ascending.
				lvwColumnSorter.SortColumn = e.Column;
				lvwColumnSorter.Order = SortOrder.Ascending;
			}

			// Perform the sort with these new sort options.
			this.listView.Sort();
		}
		
		private void OnRowClick(object sender, System.EventArgs e)
		{
			ListView.SelectedListViewItemCollection lvItems = 
				this.listView.SelectedItems;
    
			if(lvItems.Count>0)
			{
				string strItem = lvItems[0].ToString();
				strItem = strItem.Substring(strItem.IndexOf("{",0)+1);
				strItem = strItem.Substring(0,strItem.Length-1);
				strItem = strItem.Trim();

				strItem = PhotoHelper.GetPropertyDesriptionFromName(strItem);
				PhotoTips pTips = new PhotoTips();

				pTips.SetTipText(strItem);
				pTips.Show();
			}
		}

		private void LoadThumbnails()
		{
			this.panelPhotoThumbs.SuspendLayout();
			
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Loading pictures ...";
			else this.statusBarInfoPane.Text = "  ...";
			this.statusBarProgressPane.ProgressBar.Step = 100/m_arFiles.Count;
			
			int nHorizontal = -1;
			// create the thumbnails for the selected files
			foreach (string strFile in m_arFiles)
			{
				ThumbNail thumbnail = null;
				try
				{
					thumbnail = new ThumbNail(strFile);
				}
				catch (System.OutOfMemoryException)
				{
					thumbnail = null;
					continue;
				}
				if (nHorizontal < 0)
				{
					// determine how many thumbnails can be displayed on one row
					nHorizontal = (int)(this.panelPhotoThumbs.Width/(thumbnail.Width+2*m_nMargin));
				}
				// set the position for the thumbnail and add it to the panel's controls
				thumbnail.Left = m_nMargin + (thumbnail.Width+m_nMargin) * (this.panelPhotoThumbs.Controls.Count % nHorizontal);
				thumbnail.Top = m_nMargin + (thumbnail.Height+m_nMargin) * (this.panelPhotoThumbs.Controls.Count / nHorizontal);
				this.Invoke(m_DelegateAddThumbNail, new Object[] {thumbnail});
				
				this.statusBarProgressPane.ProgressBar.PerformStep();
			}

			Thread.Sleep(500);
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Ready";
			else this.statusBarInfoPane.Text = "";
			this.statusBarProgressPane.ProgressBar.Value = 0;
			this.panelPhotoThumbs.ResumeLayout();

			m_pConvertThread = null;
		}
		
		public void AddThumbNail(ThumbNail thumbnail)
		{
			this.panelPhotoThumbs.Controls.Add(thumbnail);
		}

		private void DisplayImageProperties(string strPath)
		{
			int nControls = this.panelPhotoThumbs.Controls.Count;
			if(nControls>0)
			{
				for(int i=0; i<nControls; i++)
				{
					ThumbNail thumbnail = this.panelPhotoThumbs.Controls[i] as ThumbNail;
					if(thumbnail.FilePath.Equals(strPath))
					{
						int[] propIds = thumbnail.Image.PropertyIdList;

						string strProp = "";
						string strValue = "";

						this.listView.Items.Clear();

						int nListWidth = this.listView.Width;
						this.columnHeaderTag.Width = nListWidth/3;
						this.columnHeaderValue.Width = nListWidth - this.columnHeaderTag.Width;
						this.labelPictureCaption.Text = Path.GetFileName(strPath);
						foreach(int id in propIds) 
						{
							// Since an exception can occur while accessing properties 
							// with null values we must use try-catch.
							try 
							{
								strProp = PhotoHelper.GetPropertyNameFromId(id);
								if(!strProp.Equals(""))
								{
									ListViewItem lvi = this.listView.Items.Add(strProp);
									strValue = PhotoHelper.GetPropertyValue(thumbnail.Image.GetPropertyItem(id));
									lvi.SubItems.Add(strValue);
									
									// paint label
									if(strProp.Equals("DateTimeDigitized"))
									{
										if(strValue.Substring(0,4).Equals("0000")) this.labelPictureCaption.ForeColor = Color.FromArgb(200, 0, 0);
										else this.labelPictureCaption.ForeColor = Color.FromArgb(0, 200, 0);
									}
								}
							}
							catch(Exception) 
							{
							}
						}
						int nCount = this.listView.Items.Count;
						if(nCount>0)
						{
							Rectangle rcItem = this.listView.GetItemRect(0);
							int nHeight = this.listView.ClientSize.Height;

							if(nHeight<(rcItem.Height+1)*nCount)
							{
								this.columnHeaderTag.Width = nListWidth/3;
								this.columnHeaderValue.Width = nListWidth - this.columnHeaderTag.Width - SystemInformation.VerticalScrollBarWidth - 2*SystemInformation.Border3DSize.Width;
							}
							else
							{
								this.columnHeaderTag.Width = nListWidth/3;
								this.columnHeaderValue.Width = nListWidth - this.columnHeaderTag.Width - 2*SystemInformation.Border3DSize.Width;
							}
						}
						else
						{
							this.columnHeaderTag.Width = nListWidth/3;
							this.columnHeaderValue.Width = nListWidth - this.columnHeaderTag.Width - 2*SystemInformation.Border3DSize.Width;
						}

						FileInfo finfo = new FileInfo(strPath);
						System.DateTime cdt = finfo.CreationTime;
						System.DateTime ldt = finfo.LastAccessTime;
						System.DateTime mdt = finfo.LastWriteTime;

						this.textCreation.Text = String.Format("{0:yyyy}:{0:MM}:{0:dd} {0:HH}:{0:mm}:{0:ss}",cdt);
						this.textLastWrite.Text = String.Format("{0:yyyy}:{0:MM}:{0:dd} {0:HH}:{0:mm}:{0:ss}",mdt);
						this.textLastAccess.Text = String.Format("{0:yyyy}:{0:MM}:{0:dd} {0:HH}:{0:mm}:{0:ss}",ldt);
						
						break;
					}
				}
			}

		}
		
		[System.Security.Permissions.PermissionSet(System.Security.Permissions.SecurityAction.Demand, Name="FullTrust")]
		protected override void WndProc(ref Message m) 
		{
			// Listen for operating system messages.
			switch (m.Msg)
			{
				case Win32.WM_COPYDATA:
				{
					Win32.COPYDATASTRUCT cd = (Win32.COPYDATASTRUCT)
						m.GetLParam(typeof(Win32.COPYDATASTRUCT));

					m_strFilePath = cd.lpData;

					if(!m_strFilePath.Equals(""))
					{
						DisplayImageProperties(m_strFilePath);
						menuFileInsert.Enabled = true;
						menuFileDelete.Enabled = true;
						menuFileUpdate.Enabled = true;
					}
				}
				break;
			}
			base.WndProc(ref m);
		}

		private void OnButtonChange(object sender, System.EventArgs e)
		{
			if(this.labelPictureCaption.Equals("Picture") || m_strFilePath.Equals(""))
			{
				if(m_nLanguage == 0) MessageBox.Show("Please open and select the Picture first","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				else MessageBox.Show("    ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				return;
			}

			string strDate = "";
			if(this.radioCreation.Checked) strDate = this.textCreation.Text;
			else if(this.radioLastWrite.Checked) strDate = this.textLastWrite.Text;
			else if(this.radioLastAccess.Checked) strDate = this.textLastAccess.Text;
			else if(this.radioCustomDate.Checked) strDate = this.dateCustomTimePicker.Text;
			
			if(!strDate.Equals(""))
			{
				int nControls = this.panelPhotoThumbs.Controls.Count;
				if(nControls>0)
				{
					for(int i=0; i<nControls; i++)
					{
						ThumbNail thumbnail = this.panelPhotoThumbs.Controls[i] as ThumbNail;
						if(thumbnail.FilePath.Equals(m_strFilePath))
						{
							thumbnail.UpdateDate(strDate, m_bUseDateTimeCreated, m_bUseDateTimeOriginal, m_bUseDateTimeDigitized, m_bCreateBackups);
							if(m_bUpdateFileDate) thumbnail.UpdateFileDate(strDate);
							DisplayImageProperties(m_strFilePath);
							break;
						}
					}
				}
			}
			else
			{
				if(m_nLanguage == 0) MessageBox.Show("Please select the Date Template first","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Exclamation);
				else MessageBox.Show("    ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Exclamation);
			}
		}

		private void OnOptions(object sender, System.EventArgs e)
		{
			FormOptions formOptions = new FormOptions(m_nLanguage);
			if(formOptions.ShowDialog(this).Equals(DialogResult.OK))
			{
				// attempt to read state from registry
				RegistryKey hKey = Registry.CurrentUser.OpenSubKey(@"Software\ExifChanger");
				if(hKey != null)
				{
					int nValue = 0;

					nValue = (int)hKey.GetValue("DTCreated",1);
					if(nValue<1) m_bUseDateTimeCreated = false;
					else m_bUseDateTimeCreated = true;
					
					nValue = (int)hKey.GetValue("DTDigitized",1);
					if(nValue<1) m_bUseDateTimeDigitized = false;
					else m_bUseDateTimeDigitized = true;
					
					nValue = (int)hKey.GetValue("DTOriginal",1);
					if(nValue<1) m_bUseDateTimeOriginal = false;
					else m_bUseDateTimeOriginal = true;

					nValue = (int)hKey.GetValue("Backups",1);
					if(nValue<1) m_bCreateBackups = false;
					else m_bCreateBackups = true;

					hKey.Close();
				}
			}
		}

		private void OnFileUpdate(object sender, System.EventArgs e)
		{
			if(this.labelPictureCaption.Equals("Picture") || m_strFilePath.Equals(""))
			{
				if(m_nLanguage == 0) MessageBox.Show("Please open and select the Picture first","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				else MessageBox.Show("    ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				return;
			}

			string strDate = "";
			if(this.radioCreation.Checked) strDate = this.textCreation.Text;
			else if(this.radioLastWrite.Checked) strDate = this.textLastWrite.Text;
			else if(this.radioLastAccess.Checked) strDate = this.textLastAccess.Text;
			else if(this.radioCustomDate.Checked) strDate = this.dateCustomTimePicker.Text;
			
			if(!strDate.Equals(""))
			{
				int nControls = this.panelPhotoThumbs.Controls.Count;
				if(nControls>0)
				{
					for(int i=0; i<nControls; i++)
					{
						ThumbNail thumbnail = this.panelPhotoThumbs.Controls[i] as ThumbNail;
						if(thumbnail.FilePath.Equals(m_strFilePath))
						{
							if(thumbnail.UpdateFileDate(strDate))
							{
								if(m_nLanguage == 0) MessageBox.Show("File Date has been updated successfully","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Information);
								else MessageBox.Show("/    ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Information);
								DisplayImageProperties(m_strFilePath);
							}
							else 
							{
								if(m_nLanguage == 0) MessageBox.Show("Cannot update File DateTime","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Error);
								else MessageBox.Show("  /  ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Error);
							}
							
							break;
						}
					}
				}
			}
			else
			{
				if(m_nLanguage == 0) MessageBox.Show("Please select the Date Template first","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Exclamation);
				else MessageBox.Show("    ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Exclamation);
			}
		}

		private void InsertDate(int nTagId)
		{
			string strDate = "";
			if(this.radioCreation.Checked) strDate = this.textCreation.Text;
			else if(this.radioLastWrite.Checked) strDate = this.textLastWrite.Text;
			else if(this.radioLastAccess.Checked) strDate = this.textLastAccess.Text;
			else if(this.radioCustomDate.Checked) strDate = this.dateCustomTimePicker.Text;
			
			if(!strDate.Equals(""))
			{
				int nControls = this.panelPhotoThumbs.Controls.Count;
				if(nControls>0)
				{
					for(int i=0; i<nControls; i++)
					{
						ThumbNail thumbnail = this.panelPhotoThumbs.Controls[i] as ThumbNail;
						if(thumbnail.FilePath.Equals(m_strFilePath))
						{
							thumbnail.InsertDate(strDate, nTagId, m_bCreateBackups, m_nLanguage);
							DisplayImageProperties(m_strFilePath);
							break;
						}
					}
				}
			}
			else 
			{
				if(m_nLanguage == 0) MessageBox.Show("Please select the Date Template first","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Exclamation);
				else MessageBox.Show("    ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Exclamation);
			}
		}

		private void DeleteDate(int nTagId)
		{
			if(this.labelPictureCaption.Equals("Picture") || m_strFilePath.Equals(""))
			{
				if(m_nLanguage == 0) MessageBox.Show("Please open and select the Picture first","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				else MessageBox.Show("    ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				return;
			}
			
			int nControls = this.panelPhotoThumbs.Controls.Count;
			if(nControls>0)
			{
				for(int i=0; i<nControls; i++)
				{
					ThumbNail thumbnail = this.panelPhotoThumbs.Controls[i] as ThumbNail;
					if(thumbnail.FilePath.Equals(m_strFilePath))
					{
						thumbnail.DeleteDate(nTagId, m_bCreateBackups, m_nLanguage);
						DisplayImageProperties(m_strFilePath);
						break;
					}
				}
			}
		}
		
		private void OnFileExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void OnInsertDTCreated(object sender, System.EventArgs e)
		{
			if(this.labelPictureCaption.Equals("Picture") || m_strFilePath.Equals(""))
			{
				if(m_nLanguage == 0) MessageBox.Show("Please open and select the Picture first","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				else MessageBox.Show("    ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				return;
			}

			DialogResult nResult = DialogResult.No;
			if(m_nLanguage == 0) nResult = MessageBox.Show("Do you want to insert new DateTime property?","EXIF Date Changer",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
			else nResult = MessageBox.Show("     DateTime ?","EXIF Date Changer",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
			if(nResult.Equals(DialogResult.Yes)) InsertDate(306);
		}

		private void OnInsertDTOriginal(object sender, System.EventArgs e)
		{
			if(this.labelPictureCaption.Equals("Picture") || m_strFilePath.Equals(""))
			{
				if(m_nLanguage == 0) MessageBox.Show("Please open and select the Picture first","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				else MessageBox.Show("    ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				return;
			}

			DialogResult nResult = DialogResult.No;
			if(m_nLanguage == 0) nResult = MessageBox.Show("Do you want to insert new DateTimeOriginal property?","EXIF Date Changer",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
			else nResult = MessageBox.Show("     DateTimeOriginal ?","EXIF Date Changer",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
			if(nResult.Equals(DialogResult.Yes)) InsertDate(36867);
		}

		private void OnInsertDTDigitized(object sender, System.EventArgs e)
		{
			if(this.labelPictureCaption.Equals("Picture") || m_strFilePath.Equals(""))
			{
				if(m_nLanguage == 0) MessageBox.Show("Please open and select the Picture first","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				else MessageBox.Show("    ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				return;
			}

			DialogResult nResult = DialogResult.No;
			if(m_nLanguage == 0) nResult = MessageBox.Show("Do you want to insert new DateTimeDigitized property?","EXIF Date Changer",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
			else nResult = MessageBox.Show("     DateTimeDigitized ?","EXIF Date Changer",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
			if(nResult.Equals(DialogResult.Yes)) InsertDate(36868);
		}

		private void OnDeleteDTCreated(object sender, System.EventArgs e)
		{
			if(this.labelPictureCaption.Equals("Picture") || m_strFilePath.Equals(""))
			{
				if(m_nLanguage == 0) MessageBox.Show("Please open and select the Picture first","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				else MessageBox.Show("    ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				return;
			}

			DialogResult nResult = DialogResult.No;
			if(m_nLanguage == 0) nResult = MessageBox.Show("Do you want to delete DateTime property?","EXIF Date Changer",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
			else nResult = MessageBox.Show("   DateTime ?","EXIF Date Changer",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
			if(nResult.Equals(DialogResult.Yes)) DeleteDate(306);
		}

		private void OnDeleteDTOriginal(object sender, System.EventArgs e)
		{
			if(this.labelPictureCaption.Equals("Picture") || m_strFilePath.Equals(""))
			{
				if(m_nLanguage == 0) MessageBox.Show("Please open and select the Picture first","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				else MessageBox.Show("    ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				return;
			}

			DialogResult nResult = DialogResult.No;
			if(m_nLanguage == 0) nResult = MessageBox.Show("Do you want to delete DateTimeOriginal property?","EXIF Date Changer",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
			else nResult = MessageBox.Show("   DateTimeOriginal ?","EXIF Date Changer",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
			if(nResult.Equals(DialogResult.Yes)) DeleteDate(36867);
		}

		private void OnDeleteDTDigitized(object sender, System.EventArgs e)
		{
			if(this.labelPictureCaption.Equals("Picture") || m_strFilePath.Equals(""))
			{
				if(m_nLanguage == 0) MessageBox.Show("Please open and select the Picture first","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				else MessageBox.Show("    ","EXIF Date Changer",MessageBoxButtons.OK,MessageBoxIcon.Warning);
				return;
			}

			DialogResult nResult = DialogResult.No;
			if(m_nLanguage == 0) nResult = MessageBox.Show("Do you want to delete DateTimeDigitized property?","EXIF Date Changer",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
			else nResult = MessageBox.Show("   DateTimeDigitized ?","EXIF Date Changer",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
			if(nResult.Equals(DialogResult.Yes)) DeleteDate(36868);
		}

		private void OnLanguageEnglish(object sender, System.EventArgs e)
		{
			m_nLanguage = 0;
			this.menuLanguageEnglish.Checked = true;
			this.menuLanguageRussian.Checked = false;
			SwitchLanguage();
		}

		private void OnLanguageRussian(object sender, System.EventArgs e)
		{
			m_nLanguage = 1;
			this.menuLanguageEnglish.Checked = false;
			this.menuLanguageRussian.Checked = true;
			SwitchLanguage();
		}

		private void OnHelpAbout(object sender, System.EventArgs e)
		{
			FormAbout formAbout = new FormAbout(m_nLanguage);
			formAbout.ShowDialog(this);
		}

		#region Language Switch
		private void SwitchLanguage()
		{
			if(m_nLanguage == 0)
			{
				this.menuFile.Text = "File";
				this.menuFileOpen.Text = "Open File";
				this.menuFileOpenFolder.Text = "Open Folder";
				this.menuFileInsert.Text = "Insert Tag";
				this.menuFileDelete.Text = "Delete Tag";
				this.menuFileUpdate.Text = "Update File Date";
				this.menuFileExit.Text = "Exit";
				this.menuSettings.Text = "Settings";
				this.menuOptions.Text = "Options";
				this.menuLanguage.Text = "Language";
				this.menuLanguageEnglish.Text = "English";
				this.menuLanguageRussian.Text = "Russian";
				this.menuHelp.Text = "Help";
				this.menuHelpAbout.Text = "About program";
				this.menuHelpManual.Text = "Manual";
				this.columnHeaderTag.Text = "  Property";
				this.columnHeaderValue.Text = "  Value";
				if(this.labelPictureCaption.Text.Equals("")) this.labelPictureCaption.Text = "Picture";
				this.radioCreation.Text = "Creation Date";
				this.radioLastAccess.Text = "Last Access Date";
				this.radioLastWrite.Text = "Last Write Date";
				this.radioCustomDate.Text = "Custom Date";
				this.buttonChange.Text = "Change";
			}
			else
			{
				this.menuFile.Text = "";
				this.menuFileOpen.Text = " ";
				this.menuFileOpenFolder.Text = " ";
				this.menuFileInsert.Text = " ";
				this.menuFileDelete.Text = " ";
				this.menuFileUpdate.Text = "  ";
				this.menuFileExit.Text = "";
				this.menuSettings.Text = "";
				this.menuOptions.Text = "";
				this.menuLanguage.Text = "";
				this.menuLanguageEnglish.Text = "";
				this.menuLanguageRussian.Text = "";
				this.menuHelp.Text = "";
				this.menuHelpAbout.Text = " ";
				this.menuHelpManual.Text = "";
				this.columnHeaderTag.Text = "  ";
				this.columnHeaderValue.Text = "  ";
				if(this.labelPictureCaption.Text.Equals("Picture")) this.labelPictureCaption.Text = "";
				this.radioCreation.Text = " ";
				this.radioLastAccess.Text = " ";
				this.radioLastWrite.Text = " ";
				this.radioCustomDate.Text = "";
				this.buttonChange.Text = " ";
			}
		}
		#endregion

		#region Select notify functions

		private void OnSelectFileOpen(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Opens an existing picture";
			else this.statusBarInfoPane.Text = "  ";
			this.timerTips.Start();
		}

		private void OnSelectFolderOpen(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Opens an existing folder with pictures";
			else this.statusBarInfoPane.Text = "   ";
			this.timerTips.Start();
		}

		private void OnSelectFileUpdate(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Updates the File DateTime value";
			else this.statusBarInfoPane.Text = " /  ";
			this.timerTips.Start();
		}
		
		private void OnSelectOptions(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Selects an options";
			else this.statusBarInfoPane.Text = " ";
			this.timerTips.Start();
		}

		private void OnSelectExit(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Quits the application";
			else this.statusBarInfoPane.Text = "  ";
			this.timerTips.Start();
		}

		private void OnSelectInsertDTCreated(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Inserts new DateTime image tag";
			else this.statusBarInfoPane.Text = "    DateTime ";
			this.timerTips.Start();
		}

		private void OnSelectInsertDTOriginal(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Inserts new DateTimeOriginal image tag";
			else this.statusBarInfoPane.Text = "    DateTimeOriginal ";
			this.timerTips.Start();
		}

		private void OnSelectInsertDTDigitized(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Inserts new DateTimeDigitized image tag";
			else this.statusBarInfoPane.Text = "    DateTimeDigitized ";
			this.timerTips.Start();
		}

		private void OnSelectDeleteDTCreated(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Deletes the DateTime image tag";
			else this.statusBarInfoPane.Text = " DateTime ";
			this.timerTips.Start();
		}

		private void OnSelectDeleteDTOriginal(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Deletes the DateTimeOriginal image tag";
			else this.statusBarInfoPane.Text = " DateTimeOriginal ";
			this.timerTips.Start();
		}

		private void OnSelectDeleteDTDigitized(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Deletes the DateTimeDigitized image tag";
			else this.statusBarInfoPane.Text = " DateTimeDigitized ";
			this.timerTips.Start();
		}

		private void OnSelectLanguage(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Selects an interface language";
			else this.statusBarInfoPane.Text = "  ";
			this.timerTips.Start();
		}

		private void OnSelectManual(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Opens the program manual";
			else this.statusBarInfoPane.Text = "  ";
			this.timerTips.Start();
		}

		private void OnSelectAbout(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Displays the program information and copyright";
			else this.statusBarInfoPane.Text = "   ";
			this.timerTips.Start();
		}
		#endregion

		private void OnTimerTips(object sender, System.EventArgs e)
		{
			this.timerTips.Stop();
			if(m_nLanguage == 0) this.statusBarInfoPane.Text = "Ready";
			else this.statusBarInfoPane.Text = "";
		}

		private void OnHelp(object sender, System.EventArgs e)
		{
			try
			{
				System.Reflection.Module[] modules = System.Reflection.Assembly.GetExecutingAssembly().GetModules();
				string strPath = System.IO.Path.GetDirectoryName (modules[0].FullyQualifiedName);
				if((strPath != "") && (strPath[strPath.Length-1] != '\\')) strPath += '\\';

				strPath += "edc.chm";

				System.Diagnostics.Process.Start(strPath);
			}
			catch (System.ComponentModel.Win32Exception noBrowser)
			{
				if(noBrowser.ErrorCode==-2147467259) MessageBox.Show(noBrowser.Message);
			}
			catch (System.Exception other)
			{
				MessageBox.Show(other.Message);
			}
		}

	}
}
